import { RedisClientOptions, RedisClientType, RedisFunctions, RedisModules, RedisScripts } from "redis";
/**
 * Possible Config parameters for SecureStore constructor
 */
export interface SecureStoreConfig {
    /**
     * A unique ID which can be used to prefix data stored in Redis
     */
    uid?: string;
    /**
     * A 32 character encryption secret, it will be automatically generated if not provided
     */
    secret?: string;
    /**
     * Redis connect config object
     */
    redis?: RedisClientOptions;
}
/**
 * SecureStore class
 *
 * Automatically encrypt any data saved to redis
 *
 * @export
 * @class SecureStore
 */
export default class SecureStore {
    /**
     * Redis client
     */
    client: RedisClientType<RedisModules, RedisFunctions, RedisScripts>;
    private readonly config;
    /**
     * Creates an instance of SecureStore.
     *
     * @constructor
     */
    constructor(cfg: SecureStoreConfig);
    /**
     * Disconnects the Redis client
     */
    disconnect(client?: RedisClientType<RedisModules, RedisFunctions, RedisScripts>): Promise<void>;
    /**
     * Initializes (connects) the Redis client to the Redis server
     */
    init(): Promise<void>;
    /**
     * Save and encrypt arbitrary data to Redis
     */
    save(key: string, data: unknown, postfix?: string): Promise<number>;
    /**
     * Get and decrypt arbitrary data from Redis
     */
    get(key: string, postfix?: string): Promise<any>;
    /**
     * Delete arbitrary data from Redis
     */
    delete(key: string, postfix?: string): Promise<number>;
    /**
     * Encrypts arbitrary data, returning an encrypted string
     */
    private encrypt;
    /**
     * Decrypts given encrypted string, returning its arbitrary data
     */
    private decrypt;
    /**
     * Generate sha256 sum from given text
     */
    private static shasum;
}
